class WcsTriggerButton extends HTMLElement {
  triggerUrl

  connectedCallback () {
    this.addEventListener('click', this.onClick.bind(this))
    const label = this.getAttribute('label')
    this.triggerUrl = this.getAttribute('trigger-url')
    this.innerHTML = `<button>${label}<i class="wcs-trigger-button--success-icon"></i></button>`
    if (this.triggerUrl === null) {
      const unavailableMode = this.getAttribute('unavailable-mode')
      if (unavailableMode === 'hide') {
        this.hidden = true
      } else {
        const button = this.querySelector('button')
        button.disabled = true
      }
    }
  }

  async onClick () {
    const confirmationMessage = this.getAttribute('confirmation-message')

    if (confirmationMessage && !confirm(confirmationMessage)) {
      return
    }

    const response = await fetch(
      this.getAttribute('proxy-url'),
      {
        method: 'POST',
        mode: 'cors',
        credentials: 'same-origin',
        headers: {
          'Content-Type': 'application/json',
          'X-CSRFToken': this.getAttribute('csrf-token'),
        },
        body: JSON.stringify({
          'trigger-url': this.triggerUrl,
        }),
      },
    )

    const json = await response.json()

    const button = this.querySelector('button')
    if (!response.ok || json.err === 1) {
      alert(this.getAttribute('error-message'))
      button.disabled = false
    } else {
      this.classList.add('success')
      button.disabled = true
    }
  }
}

window.customElements.define('wcs-trigger-button', WcsTriggerButton)

